package com.suliang.common.base.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.databinding.ViewDataBinding
import androidx.fragment.app.Fragment
import com.suliang.common.extension.initBinding

/**
 * Fragment ViewBinding基类，封装ViewDataBinding  自带懒加载
 *
 * TabLayout+ViewPager懒加载： 关联使用 tabLayout.setupWithViewPager(ViewPager),
 * 适配器使用两参构造，第二参数使用：BEHAVIOR_RESUME_ONLY_CURRENT_FRAGMENT，否则会走setUserVisibleHint
 *
 * FragmentStatePagerAdapter会调用onSaveInstanceState方法来保存Fragment的状态
 *
 * 通过add + show + hide 模式加载Fragment
 *
 * @property VB : ViewDataBinding
 */
abstract class BaseFragment<VB : ViewDataBinding> : Fragment() {

    private var _binding: VB? = null

    val binding : VB
        get() = _binding!!

    //是否加载数据
    private var isLoaded = false

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        _binding = initBinding(inflater, container)
        return super.onCreateView(inflater, container, savedInstanceState)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initFirst()
        initFragment()
    }

    override fun onResume() {
        super.onResume()
        if (!isLoaded && !isHidden) {
            loadData()
            isLoaded = true
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        _binding = null
    }

    /**
     * onViewCreated中进行的初始：多数时候可以是空实现，主要在使用VM时进行vm的初始
     */
    abstract fun initFirst()

    /**
     * onViewCreated() 后进行的Fragment初始设置
     */
    abstract fun initFragment()

    /**
     * 界面显示出来后再加载数据
     */
    abstract fun loadData()

}