package com.suliang.common.util.file

import android.os.Environment
import com.suliang.common.util.AppGlobals
import com.suliang.common.util.LogUtil
import java.io.*
import java.lang.Exception
import java.text.DecimalFormat
import java.util.zip.GZIPInputStream
import java.util.zip.GZIPOutputStream

/**
 * author suliang
 * create 2022/3/7 17:06
 * Describe: File工具类
 */
object FileUtil {

    const val SIZETYPE_B = 1 // 获取文件大小单位为B的double值

    const val SIZETYPE_KB = 2 // 获取文件大小单位为KB的double值

    const val SIZETYPE_MB = 3 // 获取文件大小单位为MB的double值

    const val SIZETYPE_GB = 4 // 获取文件大小单位为GB的double值

    /**
     * 获取应用内置存储目录文件
     */
    private fun getAppExternalFile(): File {
        val file = if (Environment.MEDIA_MOUNTED == Environment.getExternalStorageState()) {
            AppGlobals.application.getExternalFilesDir(null)
        } else {
            AppGlobals.application.filesDir
        }
        file?.let {
            if (it.exists()) file.mkdirs()
        }
        LogUtil.d("应用内置目录文件： ${file?.path}")
        return file ?: throw NullPointerException("Expression 'file' must not be null")
    }

    /**
     * 获取定义的用于保存的目录文件 [getAppExternalFile]/[saveDirName]/
     * @param saveDirName String 保存的文件目录名称
     * @return File
     */
    fun getSaveDirFile(saveDirName: String): File {
        return File(getAppExternalFile(), saveDirName).apply {
            if (!this.exists()) mkdirs()
        }
    }

    /**
     * 获取定义的用于保存的目录路径： [getAppExternalFile]/[saveDirName]/
     * @param saveDirName String 保存的文件目录名称
     * @return String
     */
    fun getSaveDirPath(saveDirName: String): String {
        return getSaveDirFile(saveDirName).path
    }

    /**
     * 创建一个新文件
     * @param parentFile File 父文件
     * @param fileName String 文件名
     * @return File 创建的文件
     */
    fun createNewFile(parentFile: File, fileName: String): File {
        if (!parentFile.exists()) parentFile.mkdirs()
        val newFile = File(parentFile, fileName)
        var createNewFile = false
        try {
            createNewFile = newFile.createNewFile()
        } catch (e: IOException) {
            e.printStackTrace()
        }
        LogUtil.d("创建的新文件结果：$createNewFile , 文件路径: ${newFile.path}")
        return newFile
    }

    /**
     * 创建一个新文件
     * @param parentFilePath File 父文件路径
     * @param fileName String 文件名
     * @return File 创建的文件
     */
    fun createNewFile(parentFilePath: String, fileName: String): File {
        return createNewFile(File(parentFilePath), fileName)
    }

    /**
     * 删除文件/文件夹
     * @param filePath String  文件路径
     */
    fun deleteFile(filePath: String) {
        deleteFile(File(filePath))
    }

    /**
     * 删除文件/文件夹
     * @param file File 被删除的文件
     */
    fun deleteFile(file: File) {
        if (file.exists()) {
            synchronized(file) {
                if (file.isDirectory) {
                    file.listFiles()?.forEach {
                        if (file.isDirectory) {
                            deleteFile(it)
                        } else {
                            it.delete()
                        }
                    }
                }
                file.delete()
            }
        }
    }

    /**
     * 将字节数组写入文件
     * @param file File 被写入的文件
     * @param content ByteArray 被写入的内容
     */
    fun writeBytesToFile(file: File, content: ByteArray) {
        try {
            file.parentFile?.let {
                if (!it.exists()) {
                    it.mkdirs()
                }
            }
            val inputStream = ByteArrayInputStream(content)
            val outputStream = FileOutputStream(file)
            writeStream(inputStream, outputStream)
        } catch (e: Exception) {
            LogUtil.e("文件写入失败： ${file.path}")
        }
    }

    /**
     * 将字节数组写入文件
     * @param parentFile File 父文件
     * @param fileName String 写入文件名称
     * @param content ByteArray 内容
     */
    fun writeBytesToFile(parentFile: File, fileName: String, content: ByteArray) {
        val file = File(parentFile, fileName)
        writeBytesToFile(file, content)
    }

    /**
     * 将字节数组写入文件
     * @param parentFilePath String 父文件路径
     * @param fileName String 写入文件名称
     * @param content ByteArray 内容
     */
    fun writeBytesToFile(parentFilePath: String, fileName: String, content: ByteArray) {
        val file = File(parentFilePath, fileName)
        writeBytesToFile(file, content)
    }

    /**
     * 写入方法
     * @param inputStream InputStream 输入流
     * @param outputStream OutputStream  输出流
     */
    private fun writeStream(inputStream: InputStream, outputStream: OutputStream) {
        try {
            var len = 0
            val buffer: ByteArray = ByteArray(1024)
            while ((inputStream.read(buffer).also { len = it }) != -1) {
                outputStream.write(buffer, 0, len)
            }
        } catch (e: Exception) {
            LogUtil.e("写入文件失败：${e.message}")
            e.printStackTrace()
        } finally {
            closeStream(inputStream)
            closeStream(outputStream)
        }
    }

    /**
     * 复制文件
     * @param srcFile File  源文件
     * @param destFile File 目标文件
     * @param deleteSrcFile Boolean 是否需要删除源文件
     */
    @Synchronized
    fun copyFile(srcFile: File, destFile: File, deleteSrcFile: Boolean): Boolean {
        //源文件部存在，直接返回false
        if (!srcFile.exists()) return false
        //目标文件父文件判断
        destFile.parentFile?.let {
            if (!it.exists()) it.mkdirs()
        }
        try {
            val inputStream = FileInputStream(srcFile)
            val outputStream = FileOutputStream(destFile)
            writeStream(inputStream, outputStream)
            if (deleteSrcFile) srcFile.delete()
            return true
        } catch (e: Exception) {
            LogUtil.e("复制文件失败")
            e.printStackTrace()
        }
        return false
    }

    /**
     * 复制文件
     * @param srcPath File  源文件路径
     * @param destPath File 目标文件路径
     * @param deleteSrcFile Boolean 是否需要删除源文件
     */
    fun copyFile(srcPath: String, destPath: String, deleteSrcFile: Boolean): Boolean {
        return copyFile(File(srcPath), File(destPath), deleteSrcFile)
    }

    /**
     * 复制raw文件到本地
     * @param rawId Int raw文件id
     * @param parentDirFile File  父文件
     * @param copyFileName String 新文件名称
     * @return Boolean  结果
     */
    fun copyRaw(rawId: Int, parentDirFile: File, copyFileName: String): Boolean {
        try {
            if (!parentDirFile.exists()) parentDirFile.mkdirs()
            val inputStream = AppGlobals.application.resources.openRawResource(rawId)
            val outputStream = FileOutputStream(File(parentDirFile, copyFileName))
            writeStream(inputStream, outputStream)
        } catch (e: Exception) {
            e.printStackTrace()
            return false
        }
        return true
    }

    /**
     * 复制asset文件进入本地
     * @param assetName String
     * @param saveFile File
     * @return Boolean 结果
     */
    fun copyAsset(assetName: String, saveFile: File): Boolean {
        saveFile.parentFile?.let {
            if (!it.exists()) it.mkdirs()
        }
        try {
            val inputStream = AppGlobals.application.assets.open(assetName)
            val outputStream = FileOutputStream(saveFile)
            writeStream(inputStream, outputStream)
        } catch (e: Exception) {
            e.printStackTrace()
            LogUtil.e("复制asset失败: $assetName")
        }
        return true
    }

    /**
     * 复制asset到本地
     * @param assetName String  asset名字
     * @param parentFilePath String  父文件夹路径
     * @param saveFileName String  复制的名称 默认为assetName
     * @return Boolean
     */
    fun copyAsset(assetName: String, parentFilePath: String, saveFileName: String = assetName): Boolean {
        return copyAsset(assetName, File(parentFilePath, saveFileName))
    }


    /**
     * 读取文件
     * @param file File 读的文件
     * @return ByteArray 返回字节数组
     */
    fun readFile(file: File): ByteArray {
        if (!file.exists()) return ByteArray(0)

        try {
            val fileInputStream = FileInputStream(file)
            val outputStream = ByteArrayOutputStream()
            var len = 0
            var byteArray = ByteArray(1024)
            while (fileInputStream.read(byteArray).also { len = it } != -1) {
                outputStream.write(byteArray, 0, len)
            }
            byteArray = outputStream.toByteArray()
            closeStream(fileInputStream)
            closeStream(outputStream)

            return byteArray
        } catch (e: Exception) {
            LogUtil.e("读取文件是失败 ： ${e.message}")
            e.printStackTrace()
        }
        return ByteArray(0)
    }


    /**
     * 关闭流
     * @param io Closeable?
     */
    private fun closeStream(io: Closeable?) {
        io?.run {
            try {
                io.close()
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }


//    /**
    // TODO: 2022/3/8  此功能需要重新定义 进行版本适配
//     * 保存图片到相册
//     * @param context
//     * @param bitmap 图片
//     * @param name    相册下保存的文件加名称
//     * @return
//     */
//    fun saveBitmapToDicm(context: Context, bitmap: Bitmap, name: String): String? {
//        //相册路径
//        val parentfile = File(
//            Environment.getExternalStorageDirectory()
//                    .toString() + File.separator + Environment.DIRECTORY_DCIM + File.separator + name
//        )
//        if (!com.sl.lib_common.filecache.LibFileUtils.isExits(parentfile)) {
//            val mkdirs = parentfile.mkdirs()
//        }
//        var file: File? = null
//        var outputStream: FileOutputStream? = null
//        try {
//            file = File(parentfile, "IMG_" + System.currentTimeMillis() + ".jpg")
//            outputStream = FileOutputStream(file)
//            bitmap.compress(Bitmap.CompressFormat.JPEG, 100, outputStream)
//        } catch (e: Exception) {
//            e.printStackTrace()
//            return null
//        } finally {
//            com.sl.lib_common.filecache.LibFileUtils.close(outputStream)
//        }
//        MediaStore.Images.Media.insertImage(context.contentResolver, bitmap, file!!.path, null)
//        val intent = Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE)
//        val uri = Uri.fromFile(file)
//        intent.data = uri
//        context.sendBroadcast(intent)
//        return file.path
//    }
    /**
     * 字符串的GZip压缩
     * @param str 待压缩的字符串
     * @return 返回压缩后的字符串
     * @throws IOException
     */
    @Throws(IOException::class)
    fun compressGZip(str: String): String {
        if (str.isEmpty()) return str
        // 创建一个新的输出流
        val out = ByteArrayOutputStream()
        // 使用默认缓冲区大小创建新的输出流
        val gzip = GZIPOutputStream(out)
        // 将字节写入此输出流
        gzip.write(str.toByteArray(charset("utf-8"))) // 因为后台默认字符集有可能是GBK字符集，所以此处需指定一个字符集
        // 使用指定的 charsetName，通过解码字节将缓冲区内容转换为字符串
        val result = out.toString("ISO-8859-1")
        closeStream(gzip)
        closeStream(out)
        return result
    }

    /**
     * 字符串的GZip解压
     * @param str 对字符串解压
     * @return 返回解压缩后的字符串
     * @throws IOException
     */
    @Throws(IOException::class)
    fun unCompressGZip(str: String): String {
        if (str.isEmpty()) return str
        // 创建一个新的输出流
        val out = ByteArrayOutputStream()
        // 创建一个 ByteArrayInputStream，使用 buf 作为其缓冲区数组
        val inputStream = ByteArrayInputStream(str.toByteArray(charset("ISO-8859-1")))
        // 使用默认缓冲区大小创建新的输入流
        val gzip = GZIPInputStream(inputStream)
        val buffer = ByteArray(256)
        var n = 0
        // 将未压缩数据读入字节数组
        while (gzip.read(buffer).also { n = it } >= 0) {
            out.write(buffer, 0, n)
        }
        // 使用指定的 charsetName，通过解码字节将缓冲区内容转换为字符串
        val result = out.toString("utf-8")
        closeStream(inputStream)
        closeStream(out)
        return result
    }

    @Throws(Exception::class)
    fun objectToBytes(data: Any): ByteArray {
        var oos: ObjectOutputStream? = null
        return try {
            val bos = ByteArrayOutputStream()
            oos = ObjectOutputStream(bos)
            oos.writeObject(data)
            bos.toByteArray()
        } finally {
            oos?.close()
        }
    }

    /**
     * 获取文件大小
     * @param file File 需要获取大小的文件
     * @return Long
     */
    fun getFileSize(file: File): Long {
        var result = 0L
        if (!file.exists()) return 0
        if (file.isDirectory) {
            file.listFiles()?.forEach {
                result += getFileSize(it)
            }
        }else{
            result += file.length()
        }
        return result
    }

   /* private fun getFileSize(file: File) : Int{
        if (file.exists()){
            LogUtil.e("文件不存在： $file")
            return 0
        }
        val fileInputStream = FileInputStream(file)
        val size = fileInputStream.available()
        closeStream(fileInputStream)
        return size
    }*/

    /**
     * 获取文件大小，默认保留两位小数,没有两位自动补0，为四舍五入模式
     * @param file File 文件
     * @param sizeType String 格式 B、KB、MB、GB
     * @return String 结果
     */
    fun getFileSize(file: File, sizeType:String) : String {
        val fileSize = getFileSize(file)
        return when(sizeType.uppercase()){
            "B" ->  DecimalFormat("#.00").format(fileSize)
            "KB" -> DecimalFormat("#.00").format(fileSize/1024.0)
            "MB" -> DecimalFormat("#.00").format(fileSize/1048576.0)
            "GB" -> DecimalFormat("#.00").format(fileSize/1073741824.0)
            else -> "too large"
        }
    }


}